"""MediaFire Client"""

from __future__ import unicode_literals

import os
import hashlib
import logging
import requests
import posixpath

from six.moves.urllib.parse import urlparse

from mediafire.api import (MediaFireApi, MediaFireApiError)
from mediafire.uploader import (MediaFireUploader, UploadSession)

# These are educated guesses
QUICK_KEY_LENGTH = 15
FOLDER_KEY_LENGTH = 13


logger = logging.getLogger(__name__)


class ApiBugWarning(Warning):
    """Warns when remote API bug is encountered"""
    pass


class MediaFireError(Exception):
    """Base class for MediaFire errors"""
    pass


class ResourceNotFoundError(MediaFireError):
    """Raised when resource referenced cannot be found"""
    pass


class NotAFolderError(MediaFireError):
    """Raised when operation expects a folder but got something else"""
    pass


class Resource(dict):
    """Base class for MediFire resources"""
    pass


class File(Resource):
    """MediaFire File resource"""
    pass


class Folder(Resource):
    """MediaFire Folder resource"""
    pass


class MediaFireClient(object):
    """Less-user-hostile MediaFire Client"""

    def __init__(self, session_token=None):
        """Initialize MediaFireClient"""

        # pass-through to HTTP client
        self.api = MediaFireApi()

        if session_token:
            self.api.set_session_token(session_token)

    def login(self, email=None, password=None, app_id=None, api_key=None):
        """Try to obtain session token"""
        session_token = self.api.user_get_session_token(
            app_id=app_id, email=email, password=password, api_key=api_key)
        logger.debug("result: %s", session_token)

        # install session token back into api client
        self.api.set_session_token(session_token)

    def get_resource_by_uri(self, uri):
        """Resolve remote path by mediafire uri

        uri -- One of:
            mf:(quickkey|folderkey[/path/to/resource]|/path/to/resource)
            /path/to/resource
        """

        tokens = urlparse(uri)
        if tokens.path.startswith("/"):
            # Use path lookup only, root=myfiles
            result = self.get_resource_by_path(tokens.path)
        elif tokens.scheme == 'mf':
            if '/' in tokens.path:
                resource_key = tokens.path.split('/')[0]
                # get root first
                parent_folder = self.get_resource_by_key(resource_key)
                if type(parent_folder) is not Folder:
                    raise NotAFolderError(resource_key)
                path = '/'.join(tokens.path.split('/')[1:])
                # perform additional lookup by path
                result = self.get_resource_by_path(
                    path, folder_key=parent_folder['folderkey'])
            else:
                result = self.get_resource_by_key(tokens.path)
        else:
            raise ValueError("MediaFire URI must start with 'mf:' or '/'")

        return result

    def get_resource_by_key(self, resource_key):
        """Get file or folder by quick_key/folder_key

        key -- quick_key or folder_key
        """

        # search for quick_key by default
        lookup_order = ["quick_key", "folder_key"]

        if len(resource_key) == FOLDER_KEY_LENGTH:
            lookup_order = ["folder_key", "quick_key"]

        resource = None

        for lookup_key in lookup_order:
            try:
                if lookup_key == "folder_key":
                    info = self.api.folder_get_info(folder_key=resource_key)
                    resource = Folder(info['folder_info'])
                elif lookup_key == "quick_key":
                    info = self.api.file_get_info(quick_key=resource_key)
                    resource = File(info['file_info'])
            except MediaFireApiError:
                # TODO: Check response code
                pass

            if resource:
                break

        if not resource:
            raise ResourceNotFoundError(resource_key)

        return resource

    def get_resource_by_path(self, path, folder_key=None):
        """Get resource by remote path

        path -- remote path
        """
        logger.debug("resolving %s", path)

        # remove empty path components
        path = posixpath.normpath(path)
        tokens = [t for t in path.split(posixpath.sep) if t != '']

        if not len(tokens):
            # request for root
            return Folder(
                self.api.folder_get_info(folder_key)['folder_info']
            )

        search_token = tokens.pop(0)

        resource = None
        node = None
        while resource is None:
            if node is not None:
                folder_key = node['folderkey']

            folders_result = self.api.folder_get_content(folder_key=folder_key)
            folders = folders_result['folder_content']['folders']
            name_map = {i['name']: i for i in folders}

            if search_token in name_map:
                if not tokens:
                    # last component resolved
                    resource = name_map[search_token]
                    break
                else:
                    # continue searching
                    node = name_map[search_token]
                    search_token = tokens.pop(0)
                    continue

            if tokens:
                # intermediate folder not found, nothing to see here
                break

            # no more tokens, we are at the last component
            files_result = self.api.folder_get_content(folder_key=folder_key,
                                                       content_type='files')
            files = files_result['folder_content']['files']
            name_map = {i['filename']: i for i in files}

            if search_token in name_map:
                resource = name_map[search_token]
                break

            if not tokens:
                break

        if resource is None:
            raise ResourceNotFoundError(path)

        if "quickkey" in resource:
            file_info = self.api.file_get_info(
                resource['quickkey'])['file_info']
            result = File(file_info)
        elif "folderkey" in resource:
            folder_info = self.api.folder_get_info(
                resource['folderkey'])['folder_info']
            result = Folder(folder_info)

        return result

    def get_folder_contents_iter(self, uri):
        """Get directory listing iterator

        uri -- mediafire URI
        """

        resource = self.get_resource_by_uri(uri)

        if type(resource) is not Folder:
            raise NotAFolderError(uri)

        folder_key = resource['folderkey']

        lookup_params = [
            {'content_type': 'folders', 'node': 'folders', 'type': Folder},
            {'content_type': 'files', 'node': 'files', 'type': File}
        ]

        for param in lookup_params:
            chunk = 0
            while True:
                chunk += 1
                content = self.api.folder_get_content(
                    content_type=param['content_type'], chunk=chunk,
                    folder_key=folder_key)['folder_content']
                if not content[param['node']]:
                    break

                for resource_info in content[param['node']]:
                    yield param['type'](resource_info)

    def create_folder(self, uri, recursive=False):
        """Create folder

        uri -- mediafire URI
        """

        uri = uri.rstrip('/')

        folder_name = posixpath.basename(uri)
        parent_uri = posixpath.dirname(uri)

        try:
            parent_node = self.get_resource_by_uri(parent_uri)
            if type(parent_node) is not Folder:
                raise NotAFolderError(parent_uri)
            parent_key = parent_node['folderkey']
        except ResourceNotFoundError:
            if recursive:
                result = self.create_folder(parent_uri, recursive=True)
                parent_key = result['folder_key']
            else:
                raise

        # We specify exact location, so don't allow duplicates
        result = self.api.folder_create(
            folder_name, parent_key=parent_key, allow_duplicate_name='no')

        logger.info("Created folder '%s' [mf:%s]",
                    result['name'], result['folder_key'])

        return self.get_resource_by_key(result['folder_key'])

    def delete_folder(self, uri, purge=False):
        """Delete folder

        uri -- MediaFire folder URI
        """

        resource = self.get_resource_by_uri(uri)

        if type(resource) is not Folder:
            raise ValueError("Folder expected, got {}".format(type(resource)))

        if purge:
            func = self.api.folder_purge
        else:
            func = self.api.folder_delete

        try:
            result = func(resource['folderkey'])
        except MediaFireApiError as err:
            if err.code == 100:
                logger.debug(
                    "Delete folder returns error 900 but folder is deleted: "
                    "http://forum.mediafiredev.com/showthread.php?129")

                result = {}
            else:
                raise

        return result

    def delete_file(self, uri, purge=False):
        """Delete file

        uri -- MediaFire file URI
        """
        resource = self.get_resource_by_uri(uri)

        if type(resource) is not File:
            raise ValueError("File expected, got {}".format(type(resource)))

        if purge:
            func = self.api.file_purge
        else:
            func = self.api.file_delete

        return func(resource['quickkey'])

    def delete_resource(self, uri, purge=False):
        """Remove file or folder

        uri -- mediafire URI
        """
        resource = self.get_resource_by_uri(uri)

        if type(resource) is File:
            result = self.delete_file(uri, purge)
        elif type(resource) is Folder:
            result = self.delete_folder(uri, purge)
        else:
            raise ValueError('Unsupported resource: {}'.format(type(resource)))

        return result

    def upload_session(self):
        """Returns upload session context manager"""
        return UploadSession(self.api)

    def _prepare_upload_info(self, source, dest_uri):
        """Prepare Upload object, resolve paths"""

        try:
            dest_resource = self.get_resource_by_uri(dest_uri)
        except ResourceNotFoundError:
            dest_resource = None

        is_fh = hasattr(source, 'read')

        folder_key = None
        name = None

        if dest_resource:
            if type(dest_resource) is File:
                folder_key = dest_resource['parent_folderkey']
                name = dest_resource['name']
            elif type(dest_resource) is Folder:
                if is_fh:
                    raise ValueError("Cannot determine target file name")
                basename = posixpath.basename(source)
                dest_uri = posixpath.join(dest_uri, basename)
                try:
                    result = self.get_resource_by_uri(dest_uri)
                    if type(result) is Folder:
                        raise ValueError("Target is a folder (file expected)")
                except ResourceNotFoundError:
                    # ok, neither a file nor folder, proceed
                    folder_key = dest_resource['folderkey']
                    name = basename
        else:
            # get parent resource
            parent_uri = '/'.join(dest_uri.split('/')[0:-1])
            result = self.get_resource_by_uri(parent_uri)
            if type(result) is not Folder:
                raise NotAFolderError("Parent component is not a folder")

            folder_key = result['folderkey']
            name = posixpath.basename(dest_uri)

        return folder_key, name

    def upload_file(self, source, dest_uri):
        """Upload src_path to dest_uri

        source -- file-like object or path to local file
        dest_uri -- MediaFire Resource URI
        """

        folder_key, name = self._prepare_upload_info(source, dest_uri)

        is_fh = hasattr(source, 'read')

        try:
            if is_fh:
                # Re-using filehandle
                fd = source
            else:
                # Handling fs open/close
                fd = open(source, 'rb')

            return MediaFireUploader(self.api).upload(fd, name, folder_key)
        finally:
            # Close filehandle if we opened it
            if fd and not is_fh:
                fd.close()

    def download_file(self, src_uri, target):
        """Download file

        src_uri -- mediafire file URI to download
        target -- download location
        """
        resource = self.get_resource_by_uri(src_uri)
        if type(resource) is not File:
            raise MediaFireError("Only files can be downloaded")

        quick_key = resource['quickkey']
        result = self.api.file_get_links(quick_key=quick_key,
                                         link_type='direct_download')
        direct_download = result['links'][0]['direct_download']

        logger.info(direct_download)

        name = resource['filename']

        target_is_filehandle = True if hasattr(target, 'write') else False

        if not target_is_filehandle:
            if (os.path.exists(target) and os.path.isdir(target)) or \
                    target.endswith("/"):
                target = os.path.join(target, name)

            if not os.path.isdir(os.path.dirname(target)):
                os.makedirs(os.path.dirname(target))

            logger.info("Downloading %s to %s", src_uri, target)

        response = requests.get(direct_download, stream=True)
        try:
            if target_is_filehandle:
                out_fd = target
            else:
                out_fd = open(target, 'wb')

            checksum = hashlib.sha256()
            for chunk in response.iter_content(chunk_size=4096):
                if chunk:
                    out_fd.write(chunk)
                    checksum.update(chunk)

            checksum_hex = checksum.hexdigest().lower()
            if checksum_hex != resource['hash']:
                raise RuntimeError("Hash mismatch ({} != {})".format(
                    resource['hash'], checksum_hex))

            logger.info("Download completed successfully")
        finally:
            if not target_is_filehandle:
                out_fd.close()

    def update_file_metadata(self, uri, filename=None, description=None,
                             mtime=None, privacy=None):
        """Update resource metadata

        uri -- MediaFire file URI
        filename -- rename file
        description -- set file description string
        mtime -- set file modification time
        privacy -- set file privacy - 'private' or 'public'
        """

        resource = self.get_resource_by_uri(uri)

        if type(resource) is not File:
            raise ValueError('Expected File, got {}'.format(type(resource)))

        result = self.api.file_update(resource['quickkey'], filename=filename,
                                      description=description,
                                      mtime=mtime, privacy=privacy)

        return result

    def update_folder_metadata(self, uri, foldername=None, description=None,
                               mtime=None, privacy=None,
                               privacy_recursive=None):
        """Update resource metadata

        uri -- MediaFire file URI
        filename -- rename file
        description -- set file description string
        mtime -- set file modification time
        privacy -- set file privacy - 'private' or 'public'
        recursive -- update folder privacy recursively
        """

        resource = self.get_resource_by_uri(uri)

        if type(resource) is not Folder:
            raise ValueError('Expected Folder, got {}'.format(type(resource)))

        result = self.api.folder_update(resource['folderkey'],
                                        foldername=foldername,
                                        description=description,
                                        mtime=mtime,
                                        privacy=privacy,
                                        privacy_recursive=privacy_recursive)

        return result



"""MediaFireUploader - API encapsulating Upload magic"""

from __future__ import unicode_literals

import os
import hashlib
import logging
import time

from collections import namedtuple

from mediafire.subsetio import SubsetIO

# Use resumable upload if file is larger than 4Mb
UPLOAD_SIMPLE_LIMIT = 4 * 1024 * 1024

# Retry resumable uploads 5 times
UPLOAD_RETRY_COUNT = 5

# Upload polling interval in seconds
UPLOAD_POLL_INTERVAL = 5

logger = logging.getLogger(__name__)  # pylint: disable=invalid-name


# pylint: disable=too-few-public-methods,too-many-arguments
class _UploadInfo(object):
    """Structure containing upload details"""

    def __init__(self, fd=None, name=None, folder_key=None, path=None,
                 hash_=None, size=None, filedrop_key=None):
        self.fd = fd
        self.name = name
        self.folder_key = folder_key
        self.path = path
        self.hash_ = hash_
        self.size = size
        self.filedrop_key = filedrop_key


class _UploadUnitInfo(object):
    """Structure containing upload unit details"""
    def __init__(self, upload_info=None, fd=None, uid=None, hash_=None):
        self.upload_info = upload_info
        self.fd = fd
        self.uid = uid
        self.hash_ = hash_


UploadResult = namedtuple('UploadResult', [
    'action', 'quickkey', 'hash_', 'filename', 'size', 'created', 'revision'
])


# pylint: enable=too-few-public-methods,too-many-arguments
class UploadSession(object):  # pylint: disable=too-few-public-methods
    """Allocate/deallocate action token automatically"""

    def __init__(self, api):
        """Initialize context manager

        api -- MediaFireApi instance
        """
        self.action_token = None
        self._api = api

    def __enter__(self):
        """Allocate action token"""
        self.action_token = self._api.user_get_action_token(
            type_="upload", lifespan=1440)['action_token']

        self._api.set_action_token(type_="upload",
                                   action_token=self.action_token)

    def __exit__(self, *exc_details):
        """Destroys action token"""
        self._api.user_destroy_action_token(action_token=self.action_token)


def decode_resumable_upload_bitmap(bitmap_node, number_of_units):
    """Decodes bitmap_node to hash of unit_id: is_uploaded

    bitmap_node -- bitmap node of resumable_upload with
                   'count' number and 'words' containing array
    number_of_units -- number of units we are uploading to
                       define the number of bits for bitmap
    """
    bitmap = 0
    for token_id in range(int(bitmap_node['count'])):
        value = int(bitmap_node['words'][token_id])
        bitmap = bitmap | (value << (0xf * token_id))

    result = {}

    for unit_id in range(number_of_units):
        mask = 1 << unit_id
        result[unit_id] = (bitmap & mask) == mask

    return result


class MediaFireUploader(object):
    """API encapsulating Upload magic"""

    def __init__(self, api):
        """Initialize MediaFireUploader

        api -- MediaFireApi instance
        """
        self._api = api

    @staticmethod
    def sha256_digest(fd):
        """Get SHA256 digest of flehandle contents

        fd - file-like object (expects exclusive access because of seeking)
        """
        fd.seek(0, os.SEEK_SET)

        sha256sum = hashlib.sha256()
        for chunk in iter(lambda: fd.read(8192), b''):
            sha256sum.update(chunk)

        fd.seek(0, os.SEEK_SET)

        return sha256sum.hexdigest().lower()

    # pylint: disable=too-many-arguments
    def upload(self, fd, name=None, folder_key=None, filedrop_key=None,
               path=None, hash_=None):
        """Upload file, returns UploadResult object

        fd -- file-like object to upload from, expects exclusive access
        name -- file name
        folder_key -- folderkey of the target folder
        path -- path to file relative to folder_key
        hash_ -- SHA256 of contents
        filedrop_key -- filedrop to use instead of folder_key
        """

        # Allow supplying stored hash
        if hash_ is None:
            logger.debug("Calculating checksum")
            hash_ = self.sha256_digest(fd)

        # Get file handle content length in the most reliable way
        fd.seek(0, os.SEEK_END)
        size = fd.tell()
        fd.seek(0, os.SEEK_SET)

        upload_info = _UploadInfo(fd=fd, name=name, folder_key=folder_key,
                                  hash_=hash_, size=size, path=path,
                                  filedrop_key=filedrop_key)

        # We cannot use resumable upload when uploading to filedrop
        if size > UPLOAD_SIMPLE_LIMIT and filedrop_key is None:
            resumable = True
        else:
            resumable = False

        # Check whether file is present
        check_result = self._api.upload_check(
            filename=upload_info.name,
            size=upload_info.size,
            hash_=upload_info.hash_,
            folder_key=upload_info.folder_key,
            filedrop_key=upload_info.filedrop_key,
            path=upload_info.path,
            resumable=resumable
            )

        upload_result = None

        folder_key = check_result.get('folder_key', None)
        if folder_key is not None:
            # We know precisely what folder_key to use, drop path
            upload_info.folder_key = folder_key
            upload_info.path = None

        if check_result['hash_exists'] == 'yes':
            # file exists somewhere in MediaFire
            if check_result['in_folder'] == 'yes' and \
                    check_result['file_exists'] == 'yes':
                # file exists in this directory
                different_hash = check_result.get('different_hash', 'no')
                if different_hash == 'no':
                    # file is already there
                    upload_result = UploadResult(
                        action=None,
                        quickkey=check_result['duplicate_quickkey'],
                        hash_=upload_info.hash_,
                        filename=name,
                        size=upload_info.size,
                        created=None,
                        revision=None
                    )

            if not upload_result:
                # different hash or in other folder
                upload_result = self._upload_instant(upload_info)

        if not upload_result:
            if resumable:
                # Provide check_result to avoid calling API twice
                upload_result = self._upload_resumable(upload_info,
                                                       check_result)
            else:
                upload_result = self._upload_simple(upload_info)

        return upload_result
    # pylint: enable=too-many-arguments

    def _poll_upload(self, upload_key, action):
        """Poll upload until quickkey is found

        upload_key -- upload_key returned by upload/* functions
        """

        quick_key = None
        while quick_key is None:
            poll_result = self._api.upload_poll(upload_key)
            logger.debug("poll(%s): %s", upload_key, poll_result)

            doupload = poll_result['doupload']
            if int(doupload['result']) != 0:
                logger.warning("result=%d", int(doupload['result']))
                break

            if doupload['fileerror'] != '':
                # TODO: we may have to handle this a bit more dramatically
                logger.warning("fileerror=%d", int(doupload['fileerror']))
                break

            if int(doupload['status']) == 99:
                quick_key = doupload['quickkey']
            else:
                logger.debug("status=%d description=%s",
                             int(doupload['status']), doupload['description'])

                time.sleep(UPLOAD_POLL_INTERVAL)

        return UploadResult(
            action=action,
            quickkey=doupload['quickkey'],
            hash_=doupload['hash'],
            filename=doupload['filename'],
            size=doupload['size'],
            created=doupload['created'],
            revision=doupload['revision']
        )

    def _upload_instant(self, upload_info):
        """Instant upload and return quickkey

        Can be used when the file is already stored somewhere in MediaFire

        upload_info -- UploadInfo object
        """

        result = self._api.upload_instant(
            upload_info.name,
            upload_info.size,
            upload_info.hash_,
            path=upload_info.path,
            folder_key=upload_info.folder_key
        )

        return UploadResult(
            action='upload/instant',
            quickkey=result['quickkey'],
            filename=result['filename'],
            revision=result['device_revision'],
            hash_=upload_info.hash_,
            size=upload_info.size,
            created=None
        )

    def _upload_simple(self, upload_info):
        """Simple upload and return quickkey

        Can be used for small files smaller than UPLOAD_SIMPLE_LIMIT

        upload_info -- UploadInfo object
        """

        upload_result = self._api.upload_simple(
            upload_info.fd,
            upload_info.name,
            folder_key=upload_info.folder_key,
            filedrop_key=upload_info.filedrop_key,
            path=upload_info.path,
            file_size=upload_info.size,
            file_hash=upload_info.hash_)

        logger.debug("upload_result: %s", upload_result)

        upload_key = upload_result['doupload']['key']

        return self._poll_upload(upload_key, 'upload/simple')

    def _upload_resumable_unit(self, upload_unit_info):
        """Upload a single unit and return raw upload/resumable result

        unit_id -- ID of the current unit
        """

        # Get actual unit size
        unit_size = upload_unit_info.fd.len

        if upload_unit_info.hash_ is None:
            # Calculate checksum of the unit
            upload_unit_info.hash_ = self.sha256_digest(upload_unit_info.fd)

        return self._api.upload_resumable(
            upload_unit_info.fd,
            upload_unit_info.upload_info.size,
            upload_unit_info.upload_info.hash_,
            upload_unit_info.hash_,
            upload_unit_info.uid,
            unit_size,
            filedrop_key=upload_unit_info.upload_info.filedrop_key,
            folder_key=upload_unit_info.upload_info.folder_key,
            path=upload_unit_info.upload_info.path)

    def _upload_resumable_all(self, upload_info, bitmap,
                              number_of_units, unit_size):
        """Prepare and upload all resumable units and return upload_key

        upload_info -- UploadInfo object
        bitmap -- bitmap node of upload/check
        number_of_units -- number of units requested
        unit_size -- size of a single upload unit in bytes
        """

        fd = upload_info.fd

        upload_key = None

        for unit_id in range(number_of_units):
            upload_status = decode_resumable_upload_bitmap(
                bitmap, number_of_units)

            if upload_status[unit_id]:
                logger.debug("Unit %d of %d already uploaded, skipping",
                             unit_id + 1, number_of_units)
                continue

            logger.debug("Uploading unit %d of %d",
                         unit_id + 1, number_of_units)

            offset = unit_id * unit_size

            with SubsetIO(fd, offset, unit_size) as unit_fd:

                unit_info = _UploadUnitInfo(upload_info=upload_info,
                                            fd=unit_fd,
                                            uid=unit_id)

                upload_result = self._upload_resumable_unit(unit_info)

                # upload_key is needed for polling
                if upload_key is None:
                    upload_key = upload_result['doupload']['key']

        return upload_key

    def _upload_resumable(self, upload_info, check_result):
        """Resumable upload and return quickkey

        upload_info -- UploadInfo object
        check_result -- dict of upload/check call result
        """

        resumable_upload = check_result['resumable_upload']

        unit_size = int(resumable_upload['unit_size'])
        number_of_units = int(resumable_upload['number_of_units'])

        logger.debug("Uploading %d units %d bytes each",
                     number_of_units, unit_size)

        upload_key = None
        retry_count = 0

        all_units_ready = resumable_upload['all_units_ready'] == 'yes'
        bitmap = resumable_upload['bitmap']

        while not all_units_ready and retry_count < UPLOAD_RETRY_COUNT:
            logger.debug("Attempt #%d", retry_count + 1)

            upload_key = self._upload_resumable_all(upload_info, bitmap,
                                                    number_of_units, unit_size)

            check_result = self._api.upload_check(
                filename=upload_info.name,
                size=upload_info.size,
                hash_=upload_info.hash_,
                folder_key=upload_info.folder_key,
                filedrop_key=upload_info.filedrop_key,
                path=upload_info.path,
                resumable='yes')

            resumable_upload = check_result['resumable_upload']
            all_units_ready = resumable_upload['all_units_ready'] == 'yes'
            bitmap = resumable_upload['bitmap']

            if not all_units_ready:
                logger.debug("Not all units uploaded")
                retry_count += 1

        logger.debug("Upload complete. polling for status")

        return self._poll_upload(upload_key, 'upload/resumable')



from distutils.core import setup
from pip.req import parse_requirements

requirements = parse_requirements('requirements.txt')
install_requires = [str(r.req) for r in requirements]

setup(
    name='mediafire',
    version='0.2.0',
    author='Roman Yepishev',
    author_email='rye@keypressure.com',
    packages=['mediafire'],
    url='https://github.com/roman-yepishev/mediafire-python-open-sdk',
    license='BSD',
    description='Python MediaFire client library',
    long_description=open('README.rst').read(),
    install_requires=install_requires,
    keywords="mediafire sdk storage api",
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'License :: OSI Approved :: BSD License'
    ]
)
